@@ -159,7 +159,12 @@ impl Build {
             configure.arg("no-legacy");
         }
 
-        if cfg!(not(feature = "weak-crypto")) {
+        if cfg!(feature = "weak-crypto") {
+            configure
+                .arg("enable-md2")
+                .arg("enable-rc5")
+                .arg("enable-weak-ssl-ciphers");
+        } else {
             configure
                 .arg("no-md2")
                 .arg("no-rc5")
@@ -280,6 +285,7 @@ impl Build {
             "thumbv7a-uwp-windows-msvc" => "VC-WIN32-ARM-UWP",
             "x86_64-apple-darwin" => "darwin64-x86_64-cc",
             "x86_64-linux-android" => "linux-x86_64",
+            "x86_64-linux" => "linux-x86_64",
             "x86_64-pc-windows-gnu" => "mingw64",
             "x86_64-pc-windows-msvc" => "VC-WIN64A",
             "x86_64-unknown-freebsd" => "BSD-x86_64",
@@ -423,6 +429,44 @@ impl Build {
                 configure.arg("-D__STDC_NO_ATOMICS__");
             }
 
+            if target.contains("wasi") {
+                configure.args([
+                    // Termios isn't available whatsoever on WASM/WASI so we disable that
+                    "no-ui-console",
+                    // WASI doesn't support UNIX sockets so we preemptively disable it
+                    "no-sock",
+                    // WASI doesn't have a concept of syslog, so we disable it
+                    "-DNO_SYSLOG",
+                    // WASI doesn't support (p)threads. Disabling preemptively.
+                    "no-threads",
+                    // WASI/WASM aren't really friends with ASM, so we disable it as well.
+                    "no-asm",
+                    // Disables the AFALG engine (AFALG-ENGine)
+                    // Since AFALG depends on `AF_ALG` support on the linux kernel side
+                    // it makes sense that we can't use it.
+                    "no-afalgeng",
+                    "-DOPENSSL_NO_AFALGENG=1",
+                    // wasm lacks signal support; to enable minimal signal emulation, compile with
+                    // -D_WASI_EMULATED_SIGNAL and link with -lwasi-emulated-signal
+                    // The link argument is output in the `Artifacts::print_cargo_metadata` method
+                    "-D_WASI_EMULATED_SIGNAL",
+                    // WASI lacks process-associated clocks; to enable emulation of the `times` function using the wall
+                    // clock, which isn't sensitive to whether the program is running or suspended, compile with
+                    // -D_WASI_EMULATED_PROCESS_CLOCKS and link with -lwasi-emulated-process-clocks
+                    // The link argument is output in the `Artifacts::print_cargo_metadata` method
+                    "-D_WASI_EMULATED_PROCESS_CLOCKS",
+                    // WASI lacks a true mmap; to enable minimal mmap emulation, compile
+                    // with -D_WASI_EMULATED_MMAN and link with -lwasi-emulated-mman
+                    // The link argument is output in the `Artifacts::print_cargo_metadata` method
+                    "-D_WASI_EMULATED_MMAN",
+                    // WASI lacks process identifiers; to enable emulation of the `getpid` function using a
+                    // placeholder value, which doesn't reflect the host PID of the program, compile with
+                    // -D_WASI_EMULATED_GETPID and link with -lwasi-emulated-getpid
+                    // The link argument is output in the `Artifacts::print_cargo_metadata` method
+                    "-D_WASI_EMULATED_GETPID",
+                ]);
+            }
+
             if target.contains("musl") {
                 // Hack around openssl/openssl#7207 for now
                 configure.arg("-DOPENSSL_NO_SECURE_MEMORY");
@@ -490,21 +534,25 @@ impl Build {
 
     fn run_command(&self, mut command: Command, desc: &str) {
         println!("running {:?}", command);
-        let status = command.status().unwrap();
-        if !status.success() {
-            panic!(
-                "
+        let status = command.status();
+
+        let (status_or_failed, error) = match status {
+            Ok(status) if status.success() => return,
+            Ok(status) => ("Exit status", format!("{}", status)),
+            Err(failed) => ("Failed to execute", format!("{}", failed)),
+        };
+        panic!(
+            "
 
 
 Error {}:
     Command: {:?}
-    Exit status: {}
+    {}: {}
 
 
     ",
-                desc, command, status
-            );
-        }
+            desc, command, status_or_failed, error
+        );
     }
 }
 
@@ -573,6 +621,11 @@ impl Artifacts {
         println!("cargo:lib={}", self.lib_dir.display());
         if self.target.contains("msvc") {
             println!("cargo:rustc-link-lib=user32");
+        } else if self.target == "wasm32-wasi" {
+            println!("cargo:rustc-link-lib=wasi-emulated-signal");
+            println!("cargo:rustc-link-lib=wasi-emulated-process-clocks");
+            println!("cargo:rustc-link-lib=wasi-emulated-mman");
+            println!("cargo:rustc-link-lib=wasi-emulated-getpid");
         }
     }
 }
